#ifndef XICOM_UDP_HPP
#define	XICOM_UDP_HPP

#include <windows.h>
#include <winsock.h>
#include <conio.h>
#include <stdio.h>

#define XICOM_UDP_PORT			2100		// UDP port used by Xicom UDP-based protocol.
#define POLL_TIMEOUT_COUNT	6			// aka 3 seconds @ 2/sec.


extern int    system_id;	// Can be 1-15.  Matches user-selected system id for the controller/HPAs.
extern SOCKET udp_socket;


typedef unsigned char BYTE;
enum e_switch_enum 	// for clarity when indexing zero-based arrays: ES.sw[], ES.sw_inh[], etc.
	{ SWITCH1, SWITCH2, SWITCH3, SWITCH4, SWITCH5, SWITCH6, SWITCH7, SWITCH8 };
enum e_sw_priority	// switchover priority.
	{ PRIORITY_OFF, PRIORITY_AMP_A, PRIORITY_AMP_B };
enum e_event		  
	{ EVENT_DEFAULT, EVENT_FAULT, EVENT_ALARM };


/////////////////////////////// Expansion controller ////////////////////////////////
enum exc_fault_enum	// all defined expansion controller alarms/faults.
	{
	EXC_NO_ALARMS,
	A_CHECKSUM, A_BATTERY, 
	A_VPCFAN, A_SUPPLY1, A_SUPPLY2, A_UPCONV1, A_UPCONV2,	// future
	TOTAL_EXC_FAULTS
	};
struct s_exc_status
	{
	char  serial[10];			// 9 digit left-justified unit serial #, including null terminator.
	int   position;			// System position ('A', 'B' etc) returned via UDP in unit's STP; used when sending UDP commands.
	int	connected;			// Should decrement 2/sec.  If just received a response, connected = POLL_TIMEOUT_COUNT.
	BOOL  was_connected;
	sockaddr ip_address;		// Per udp packet.  Used to target the unit if we command it.
	BOOL  remotemode;
	BOOL  automode;
	BOOL  cmd_ack;				// If it claims to have processed a UDP command within last 1 second.
	BYTE  sw[8];				// 8 wg switches (using 2 for now).
	BOOL  sw_inh[8];			// 1 inhibit per wg switch.
	// for future configurations....
	BYTE  insw[2];				// 2 input switches.
	BYTE  xfer[4];				// 4 transfer switches.
	BYTE  vpc[3];				// 3 VPCs (future).
	BOOL  upcon_enabled[2];
	BOOL  ssa_inhibited[2];
	BOOL  faults[TOTAL_EXC_FAULTS];
	};
extern s_exc_status exc_state;


/////////////////////////////// Amplifiers ////////////////////////////////
enum hpa_fault_enum 	 // all possible amp alarms/faults.
	{
	NOT_FAULTED,
	A_BUC_OVERDRIVE, F_LOWLINE, F_OVERVOLT, F_UNDERVOLT, F_OVERTEMP,
	F_MAXPWR, F_MINPWR, F_RFLPWR, F_FAST_VSWR, F_SW_OVERDRIVE, F_HELIX_ARC,
	F_HELIX_ARC_L, F_WG_ARC, F_UPCONVERTER, F_IN_OVERDRIVE, F_IN_OVERDRIVE_L,
	F_EXT_INTERLOCK, F_TWT_TH_ILOCK, F_COVER_INTERLOCK, F_HEATER, F_24V,
	F_EXT_REF, F_EXT_REF_LOCK, F_LINEARIZER, F_AUX_SUPPLY, F_WG_PRESSURE,
	A_LOW_DCV, A_MAXPWR, A_MINPWR, A_RFLPWR, A_REDUCE_DRIVE, A_RAISE_DRIVE,
	A_TEMPERATURE, A_EXT_REF, A_OVERDRIVE, F_PWR_SUPPLY, 
	F_PWR_SUPPLY2, F_CABLE_INTERLOCK2,													// 2kw?
	F_SSPA_5V, F_SSPA_5V2, F_SSPA_7V, F_SSPA_7V2, F_SSPA_10V, F_SSPA_10V2, 	// sspa only.
	UNKNOWN_FAULT, TOTAL_HPA_FAULTS
	};
#define MAX_SUB_BANDS	4
#define SUB_BAND_LENGTH	7	// XX.XXX plus null terminator

struct s_amp_status
	{
	char  serial[10];		// 9 digit left-justified unit serial #, including null terminator.
	int	tubetype;		// 'T'=twta, 'K'=kpa, 'S'=sspa
	int   position;		// system position ('A', 'B' etc) returned via UDP in amp's STP; used when commanding amp via UDP.
	int	connected;		// Decremented periodically.  If just received a response, connected = POLL_TIMEOUT_COUNT.
	BOOL  was_connected;
	sockaddr ip_address;	// ip address of responding unit.
	BOOL  remotemode;		// true = remote, false = local. 
	BOOL  automode;		// true = automatic switchover enabled.
	BOOL  cmd_ack;			// "command processed within last 1 second".
	BOOL  flog_changed;	// if faultlog has changed since last faultlog query.
	float fpower;			// forward power 			dBm
	float rpower;			// reflected power 		dBm
	float temp;				// twt temperature  		C
	float helix_i;			// helix current 	 		mA
	float helix_v;			// helix voltage			kVolts
	float heater_v;		// heater voltage			volts
	float atten;			// attenuation setpoint dBm
	BOOL  has_atten;		// whether amp is configured for an attenuator.  Inferred by presence 'A' parameter in amp broadcast.
	float cpwr_targ;		// const pwr target		dBm
	BOOL  cpwr_en;			// const pwr enabled		
	BOOL  cpwr_suspend;	// const pwr suspended
	float upc_target;		// uplink power control	dBm	(if applicable)
	BOOL  upc_enabled;	// TRUE if user has enabled UPC.
	BOOL  upc_suspended;	// TRUE if UPC is off for miscellaneous reason - other than Beacon dropout.
	BOOL  ftd;
	BOOL  sbuc_warmup;	
	BOOL  hv_selected;
	BOOL  hv_on;
	BOOL  htr_standby;
	BOOL  rf_inhibited;	// hardware inhibit indicator
	BOOL  inh_selected;	// rf inhibit enabled as a user option.
	BOOL  ext_inhibit;	// external inhibit discrete command present.
	BOOL  odrive_inhibit;// inhibited by overdrive protection circuit.
	BOOL  wg_inhibit;		// inhibited by waveguide arc.
	BOOL  sw1_inhibit;	// external hardwired inhibit for wg sw1
	BOOL  sw2_inhibit;	// external hardwired inhibit for wg sw2
	DWORD h_hours;   		// heater hours. 
	DWORD b_hours;   		// beam hours.
	BOOL  sumfault;  		// summary fault.
	BOOL  sumalarm;		// summary alarm.
	int   sw1, sw2;		// waveguide switch status.
	BOOL faults[TOTAL_HPA_FAULTS];	// list of all possible faults.  Use hpa_fault_enum to index.

	float usr_max_limit;	// max forward power trip threshold: 		dBm
	float usr_min_limit;	// min forward power trip threshold: 		dBm
	float usr_rfl_limit;	// max reverse power trip threshold: 		dBm
	e_event usr_max_trip;// user trip type: alarm, fault, or off.
	e_event usr_min_trip;
	e_event usr_rfl_trip;

	float power_floor;     			// dbm.  Lowest calibrated fwd power.  Inferred when '<' returned in reading.
	float rvs_floor;					// dbm.  Lowest readable rev power.
	BOOL  batt_mv;						// SRAM backup battery voltage reading in millivolts.  If supported by unit.
	BOOL  enable_cathode_i;			// if unit supports cathode current reading.
	float	cathode_current;			// cathode current reading in mA.
	int	sub_band;	  				// selected sub-band.  0=none, legal sub-bands are 1-4.
	int	sub_bands;					// legal sub-bands.  Each bit = if that sub-band is supported.  EX: bit0=0x01=subband#1.
	char sub_band_ghz[MAX_SUB_BANDS][SUB_BAND_LENGTH];		// v20: null terminated sub-band frequency strings in Ghz (response to setup query).

	BOOL  use_1for2_options;		// if the amp supports the following extra 1:2 options.
	int   fixed_gain_switchover;	// 1 = no goto power during 1:2 switchover.  0 = execute goto power at switchover.
	float pathA_att;					// absolute attenuation dBx100, for amp-C to activate upon replacing A.  0 = don't change att.
	float	pathB_att;					// absolute attenuation dBx100, for amp-C to activate upon replacing B.  0 = don't change att.
	float	pathA_delta;				// Additional gain to apply to gotopower/UPC when amp-C replaces amp-A.
	float	pathB_delta;				// Additional gain to apply to gotopower/UPC when amp-C replaces amp-B.
	enum  e_sw_priority switchover_priority;		// 1:2 switchover priority, if any.
	};
enum amp_enum { AMP_A, AMP_B, AMP_C, TOTAL_AMPS };	// indexes to amp structs.
extern s_amp_status amp_state[TOTAL_AMPS];				// array of amp structures.






/////////////////////////////// Functions ////////////////////////////////

#define DigitToHex(i) ( (char) ((i<10)? (i+'0'): (i-10+'A')) )

// from XicomUdp.cpp:
void InitStatus();
void InitUdp();
void CloseUdp();
void CheckUdp();
void TrackConnectivity();
int  ReadUdpPacket(char *buf, int len, sockaddr *ip_addr);

// from Commands.cpp
void SendUdpCmd(sockaddr *ip, int unit_type, int position, char *serno, char *cmd);
void SetHpaInhibit(int hpa, BOOL inhibit_on);
void SetExcWgSwitch(int sw, int pos);


// from Parse.cpp:
void ParseUdpMsg(char *msg, sockaddr *ip_addr);
void ParseExpCtlStatus(char *buf, sockaddr *ip_addr);
void ParseUdpStatusAmp(char *buf, sockaddr *ip_addr);



#endif